/*
 * gpio_func.c
 *
 * API functions to control CPLD via general pin i/o.
 * Includes "init_hardware()" to setup Mainclock and configure GPIO-Pins.
 * This version is used only with prototype-hardware 0.9PT
 *
 *  Created on: 21.03.2009
 *      Author: Jan Alte, DO1FJN
 *
 *
 * This file is part of the DV-modem firmware (DVfirmware).
 * For general infomation about DVfirmware see "dv-main.c".
 *
 * DVfirmware is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * The DVfirmware is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this package. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifdef HW09PT

#include "gpio_func.h"
#include "hw_def09.h"
#include "compiler.h"


__inline void gpio0_set(unsigned int pin) {
  AVR32_GPIO.port[0].ovrs = 1 << pin;
}

__inline void gpio0_clr(unsigned int pin) {
  AVR32_GPIO.port[0].ovrc = 1 << pin;
}

__inline void gpio0_tgl(unsigned int pin) {
  AVR32_GPIO.port[0].ovrt = 1 << pin;
}

__inline void gpio1_set(unsigned int pin) {
  AVR32_GPIO.port[1].ovrs = 1 << (pin&0x1F);
}

__inline void gpio1_clr(unsigned int pin) {
  AVR32_GPIO.port[1].ovrc = 1 << (pin&0x1F);
}

__inline void gpio1_tgl(unsigned int pin) {
  AVR32_GPIO.port[1].ovrt = 1 << (pin&0x1F);
}



void cpld_start(void) {
  gpio0_clr(RUNCFG_PIN);
  AVR32_GPIO.port[0].oderc = AMBE_RESET_PIN;
//  gpio_set_gpio_open_drain_pin(AMBE_RESET_PIN);
}

void cpld_stop(void) {
  AVR32_GPIO.port[0].oders = AMBE_RESET_PIN;
  gpio0_set(RUNCFG_PIN);
//  gpio_clr_gpio_open_drain_pin(AMBE_RESET_PIN); // Reset AMBE2020
}


void cpld_config(tcpldmode mode) {
  cpld_stop();				// Switch in Config / Reset
#ifdef SSC_TFDIRECTION
  #if (SSC_TFDIRECTION==1)
  gpio1_set(CPLDTFDIR_PIN);	// Set TF Pin as Input, Internal Connect To RF
  #else
  gpio1_clr(CPLDTFDIR_PIN);	// Set TF Pin as Output, AVR32 must generate this
  #endif
#else
  gpio1_clr(CPLDTFDIR_PIN);	// Set TF Pin as Output, AVR32 must generate this
#endif
  switch (mode) {
  case CPLDdisabled:
    // This mode don't exist in Hardware V0.9PT - set to AMBE
    gpio1_clr(CPLDMODE_PIN);	// Clr Mode -> AMBE2020 Mode
    break;
  case CPLDcfg_ambeact:
  case CPLDcfg_ambepas:
    gpio1_clr(CPLDMODE_PIN);	// Clr Mode -> AMBE2020 Mode
    // AMBE2020 starts after aprox. 95ms + 8ms
    break;
  case CPLDcfg_codec:
    gpio1_set(CPLDMODE_PIN);	// Set Mode to AD73311
    break;
  };
}


__inline void watchdog(void) {
  if (Get_system_register(AVR32_COUNT)&0x00800000) {
    gpio0_set(WATCHDOG_PIN);	// toggle external Watchdog in slow freq.
  } else {
    gpio0_clr(WATCHDOG_PIN);
  }
}



void init_hardware(void) {
  // 1. Enable OSC0 with 12MHz Crystal
  AVR32_PM.oscctrl1 = (0<<AVR32_PM_OSCCTRL1_MODE_OFFSET)
    |(OSC0STARTUPVALUE<<AVR32_PM_OSCCTRL1_STARTUP_OFFSET);
  AVR32_PM.mcctrl = AVR32_PM_MCCTRL_OSC1EN_MASK;	// slow-clock + OSC0 on
  // 2. Program PLL
  AVR32_PM.pll[0] = 0x200d020F;
  // (16.384MHz*14/2 -> 114.688MHz)
  // 3. Init GPIO-Pins
  AVR32_GPIO.port[0].puer  = GPIO0_PULLUP;
  AVR32_GPIO.port[0].ovr   = GPIO0_OVRINIT;
  AVR32_GPIO.port[0].oder  = GPIO0_ODER;		// Output-Enable
  AVR32_GPIO.port[0].pmr0  = GPIO0_PMR0;
  AVR32_GPIO.port[0].pmr1  = GPIO0_PMR1;
  AVR32_GPIO.port[0].gperc = GPIO0_DISABLE_MASK;	// Disable GPIO0 control
  AVR32_GPIO.port[1].puer  = GPIO1_PULLUP;		// *** PortB (CFG, SSC, RS232) ***
  AVR32_GPIO.port[1].ovr   = GPIO1_OVRINIT;
  AVR32_GPIO.port[1].oder  = GPIO1_ODER;		// Output-Enable
  AVR32_GPIO.port[1].pmr0  = GPIO1_PMR0;
  AVR32_GPIO.port[1].pmr1  = GPIO1_PMR1;
  AVR32_GPIO.port[1].gperc = GPIO1_DISABLE_MASK;
  // 3. Set Waitstate for Flash (Clock > 30MHz)
  AVR32_FLASHC.fcr |= AVR32_FLASHC_FCR_FWS_MASK;
  // 4. Wait PLL stabilize; pm_wait_for_pll0_locked(pm);
  while (!(AVR32_PM.poscsr & AVR32_PM_POSCSR_LOCK0_MASK));
  // 5. Switch Mainclock to PLL0
  AVR32_PM.mcctrl  = AVR32_PM_MCSEL_PLL0|AVR32_PM_MCCTRL_OSC1EN_MASK;
  AVR32_PM.pbamask = PBAMASK_NEEDED;
}



#endif
