/*
 * int_func.c
 *
 * implements a simple EIC API to use external interrupt sources.
 * A simple idle timer (keep AVR32 µC alive to toggle external watchdog via main-loop)
 * is also a part of this file.
 *
 *
 *  Created on: 20.03.2009
 *      Author: DO1FJN, Jan Alte
 *
 *
 * This file is part of the DV-modem firmware (DVfirmware).
 * For general infomation about DVfirmware see "dv-main.c".
 *
 * DVfirmware is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * The DVfirmware is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this package. If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include "int_func.h"
#include "hw_defs.h"

/*! \name External Interrupt Controller Functions
 */
//! @{


void eic_enableint(unsigned int intnum, __int_handler handler, unsigned int int_lev, teictrigmode mode) {
  if (intnum < 8) {
    switch (mode) {
    case EICm_FALLING_EDGE:
      AVR32_EIC.edge &= ~(1<<intnum);
      AVR32_EIC.mode &= ~(1<<intnum);
      break;
    case EICm_RAISING_EDGE:
      AVR32_EIC.edge |= (1<<intnum);
      AVR32_EIC.mode &= ~(1<<intnum);
      break;
    case EICm_LOW_LEVEL:
      AVR32_EIC.level &= ~(1<<intnum);
      AVR32_EIC.mode |= (1<<intnum);
      break;
    case EICm_HIGH_LEVEL:
      AVR32_EIC.level |= (1<<intnum);
      AVR32_EIC.mode |= (1<<intnum);
      break;
    }
    AVR32_EIC.en = (1 << intnum);
    INTC_register_interrupt(handler, AVR32_EIC_IRQ_0+intnum, int_lev);
    AVR32_EIC.ier = (1 << intnum); 	// Int scharf
  }
}


void eic_disableint(unsigned int intnum) {
  if (intnum < 8) {
    AVR32_EIC.idr = (1 << intnum);
    AVR32_EIC.dis = (1 << intnum);
  } // fi
}


void eic_reenableint(unsigned int intnum) {
  if (intnum < 8) {
    AVR32_EIC.en = (1 << intnum);
    AVR32_EIC.ier = (1 << intnum);
  } // fi
}


void eic_clrline(unsigned int intnum) {
  Bool global_interrupt_enabled;
  // Clear line line_number
  if (intnum < 8) {
    global_interrupt_enabled = Is_global_interrupt_enabled();
    if (global_interrupt_enabled) Disable_global_interrupt();
    AVR32_EIC.icr = (1 << intnum);
    AVR32_EIC.isr;
    if (global_interrupt_enabled) Enable_global_interrupt();
  } // fi
}
//! @}


/*! \name IDLE Timer Function
 */
//! @{

#if __GNUC__
__attribute__((__interrupt__))
#elif __ICCAVR32__
__interrupt
#endif
static void idle_timer_func(void) {	// Weckt den µC aus Sleep, Watchdog in Mainloop
  AVR32_TC.channel[IDLE_TIMER_CH].sr;	// Read Status to acknowledge INT
}



void idle_timer_start(void) {
  // *** Timer: Periodisch, Up bis RC-Compare, AutoTrg ***
  AVR32_TC.channel[IDLE_TIMER_CH].cmr =  AVR32_TC_NONE << AVR32_TC_BSWTRG_OFFSET |
    AVR32_TC_NONE << AVR32_TC_BEEVT_OFFSET |
    AVR32_TC_NONE << AVR32_TC_BCPC_OFFSET |
    AVR32_TC_NONE << AVR32_TC_BCPB_OFFSET |
    AVR32_TC_NONE << AVR32_TC_ASWTRG_OFFSET |
    AVR32_TC_NONE << AVR32_TC_AEEVT_OFFSET |
    AVR32_TC_NONE << AVR32_TC_ACPC_OFFSET |
    AVR32_TC_NONE << AVR32_TC_ACPA_OFFSET |
    1 << AVR32_TC_WAVE_OFFSET |
    AVR32_TC_WAVSEL_UP_NO_AUTO << AVR32_TC_WAVSEL_OFFSET |	// Up, ohne RC-Trigger
    FALSE << AVR32_TC_ENETRG_OFFSET |
    AVR32_TC_EEVT_TIOB_INPUT << AVR32_TC_EEVT_OFFSET |
    AVR32_TC_EEVTEDG_NO_EDGE << AVR32_TC_EEVTEDG_OFFSET |
    FALSE << AVR32_TC_CPCDIS_OFFSET |
    FALSE << AVR32_TC_CPCSTOP_OFFSET |
    AVR32_TC_BURST_NOT_GATED << AVR32_TC_BURST_OFFSET |
    0 << AVR32_TC_CLKI_OFFSET |
    AVR32_TC_TCCLKS_TIMER_CLOCK5 << AVR32_TC_TCCLKS_OFFSET;	// MASTERCLK/128

  INTC_register_interrupt(idle_timer_func, IDLE_TIMER_IRQ, AVR32_INTC_INT0);
  AVR32_TC.channel[IDLE_TIMER_CH].ier = AVR32_TC_COVFS_MASK;	// Counter Overflow
  AVR32_TC.channel[IDLE_TIMER_CH].ccr = AVR32_TC_SWTRG_MASK | AVR32_TC_CLKEN_MASK;
}


void idle_timer_stop(void) {
  AVR32_TC.channel[IDLE_TIMER_CH].idr = 0xFF;
  AVR32_TC.channel[IDLE_TIMER_CH].ccr = AVR32_TC_CLKDIS_MASK;
  AVR32_TC.channel[IDLE_TIMER_CH].sr;
}


//! @}
