/*
 * rs232_func.c
 *
 * Simple RS232 communication routines using a fifo buffer.
 *
 *
 *  Created on: 11.08.2009
 *      Author: Jan Alte, DO1FJN
 *
 *
 * This file is part of the DV-modem firmware (DVfirmware).
 * For general infomation about DVfirmware see "dv-main.c".
 *
 * DVfirmware is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * The DVfirmware is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this package. If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include "rs232_func.h"
#include "hw_defs.h"
#include "gpio_func.h"		// !FORCEOFF Pin
#include "intc.h"
#include "compiler.h"
#include <string.h>


volatile U8 rs232_rxbuffer[RS232_RXBUFFERSIZE];
int rs232_rdpos;

#define rs232_wrpos	(RS232_RXBUFFERSIZE-AVR32_PDCA.channel[RS232_RXCH].tcr)

#define rs232_rxlen	((2*RS232_RXBUFFERSIZE-AVR32_PDCA.channel[RS232_RXCH].tcr-rs232_rdpos)%RS232_RXBUFFERSIZE)

/*! \name RS232 Interrupt Functions
 */
//! @{

void rs232_reset_receive(void) {
  AVR32_PDCA.channel[RS232_RXCH].cr  = AVR32_PDCA_ECLR_MASK|AVR32_PDCA_TDIS_MASK;
  AVR32_PDCA.channel[RS232_RXCH].mar = (U32)rs232_rxbuffer;
  AVR32_PDCA.channel[RS232_RXCH].tcr = sizeof(rs232_rxbuffer);
  AVR32_PDCA.channel[RS232_RXCH].marr = (U32)rs232_rxbuffer;
  AVR32_PDCA.channel[RS232_RXCH].tcrr = sizeof(rs232_rxbuffer);
  AVR32_PDCA.channel[RS232_RXCH].cr  = AVR32_PDCA_TEN_MASK;
}


#if __GNUC__
__attribute__((__interrupt__))
#elif __ICCAVR32__
__interrupt
#endif
static void rs232_pdac_reload_int(void) {
  AVR32_PDCA.channel[RS232_RXCH].marr = (U32)rs232_rxbuffer;
  AVR32_PDCA.channel[RS232_RXCH].tcrr = sizeof(rs232_rxbuffer);
}

//! @}


/*! \name RS232 API Functions
 */
//! @{

void rs232_init(unsigned long baudrate, tFlowCtrl flow) {
  RS232.cr = AVR32_USART_CR_RXDIS_MASK|AVR32_USART_CR_TXDIS_MASK|
    AVR32_USART_CR_STPBRK_MASK|AVR32_USART_CR_RSTRX_MASK|AVR32_USART_CR_RSTTX_MASK;
  rs232_auto_on();
  // Init RX-Structure:
  rs232_rdpos = 0;
  AVR32_PDCA.channel[RS232_TXCH].mr  = 0x00 << AVR32_PDCA_SIZE_OFFSET;	// Byte Transfer
  AVR32_PDCA.channel[RS232_TXCH].psr = RS232PIDTX;
  AVR32_PDCA.channel[RS232_TXCH].tcrr = 0;				// No Next Buffer
  AVR32_PDCA.channel[RS232_RXCH].mr  = 0x00 << AVR32_PDCA_SIZE_OFFSET;	// Byte Transfer
  AVR32_PDCA.channel[RS232_RXCH].psr = RS232PIDRX;
  rs232_reset_receive();
  AVR32_PDCA.channel[RS232_RXCH].ier = AVR32_PDCA_RCZ_MASK;
  INTC_register_interrupt(&rs232_pdac_reload_int, AVR32_PDCA_IRQ_0+RS232_RXCH, AVR32_INTC_INT0);

  RS232.mr = (AVR32_USART_MR_NBSTOP_1 << AVR32_USART_MR_NBSTOP_OFFSET) | // 1 StopBits
    (AVR32_USART_MR_PAR_NONE << AVR32_USART_MR_PAR_OFFSET) |	// no Parity
    ((8 - 5) << AVR32_USART_MR_CHRL_OFFSET) |	// char length = 8
    (AVR32_USART_MR_MODE_NORMAL << AVR32_USART_MR_MODE_OFFSET);	// automatic Echo
  RS232.brgr = (MASTERCLOCK+(baudrate<<3))/(baudrate<<4);
  RS232.rtor = RS232_TIMEOUT;	// Receive TimeOut Register
  RS232.cr  = AVR32_USART_CR_RXEN_MASK|AVR32_USART_CR_TXEN_MASK| \
    AVR32_USART_CR_RSTSTA_MASK|AVR32_USART_CR_STTTO_MASK;
  // Reset Status im CSR & Start Waiting TimeOut (löscht TIMEOUT Bit in CSR)
}


void rs232_exit(void) {
  AVR32_PDCA.channel[RS232_RXCH].idr = 0xFF;
  AVR32_PDCA.channel[RS232_RXCH].cr  = AVR32_PDCA_ECLR_MASK|AVR32_PDCA_TDIS_MASK;
  RS232.idr = AVR32_USART_IDR_TIMEOUT_MASK;
  RS232.cr = AVR32_USART_CR_RXDIS_MASK|AVR32_USART_CR_TXDIS_MASK|
    AVR32_USART_CR_STPBRK_MASK|AVR32_USART_CR_RSTRX_MASK|AVR32_USART_CR_RSTTX_MASK;
  rs232_force_off();
}


void rs232_settimeout(unsigned long baudrate, unsigned int ms) {
  RS232.rtor = ((baudrate*ms)+500)/1000;	// Receive TimeOut Register
}


void rs232_transmit(const char *data, int len) {
  if (len==0) return;
  if (AVR32_PDCA.channel[RS232_TXCH].sr&AVR32_PDCA_TEN_MASK) {
    if (AVR32_PDCA.channel[RS232_TXCH].tcr==0) {	// Nothing to TX
      AVR32_PDCA.channel[RS232_TXCH].mar = (U32)data;
      AVR32_PDCA.channel[RS232_TXCH].tcr = len;
    } else {
      AVR32_PDCA.channel[RS232_TXCH].marr = (U32)data;
      AVR32_PDCA.channel[RS232_TXCH].tcrr = len;
    }
  } else { // else not enabled
    AVR32_PDCA.channel[RS232_TXCH].cr  = AVR32_PDCA_ECLR_MASK|AVR32_PDCA_TDIS_MASK;
    AVR32_PDCA.channel[RS232_TXCH].mar = (U32)data;
    AVR32_PDCA.channel[RS232_TXCH].tcr = len;
    AVR32_PDCA.channel[RS232_TXCH].tcrr = 0;
    AVR32_PDCA.channel[RS232_TXCH].cr  = AVR32_PDCA_TEN_MASK;
  }
}



__inline void rs232_flushrx(void) {
  rs232_rdpos = rs232_wrpos;
}


__inline int rs232_received(void) {
  return rs232_rxlen;
}


int rs232_copyblock(char *destbuffer, int len) {
  char *nxptr;
  if (len <= RS232_RXBUFFERSIZE) {
    if ((rs232_rdpos+len) <= RS232_RXBUFFERSIZE) {	// Normalfall: kein Wrap
      memcpy(destbuffer, (const char *)&rs232_rxbuffer[rs232_rdpos], len);
    } else {
      nxptr = memcpy(destbuffer, (const char *)&rs232_rxbuffer[rs232_rdpos], RS232_RXBUFFERSIZE-rs232_rdpos);
      len  -= (RS232_RXBUFFERSIZE-rs232_rdpos);
      memcpy(nxptr, (const char *)rs232_rxbuffer, len);
    }
    rs232_rdpos = (rs232_rdpos+len)%RS232_RXBUFFERSIZE;
    return len;
  } else return 0;
}

//! @}

